from django.test import TestCase

# from tom_fink.fink import FinkQueryForm, FinkBroker


class TestFinkQueryForm(TestCase):
    """
    NOTE: to run these tests in your venv: python ./tom_fink/tests/run_tests.py
    """
    def setUp(self):
        pass

    def test_boilerplate(self):
        """Make sure the testing infrastructure is working."""
        self.assertTrue(True)


class TestFinkBrokerClass(TestCase):
    """NOTE: To run these tests in your venv: python ./tom_fink/tests/run_tests.py"""

    def setUp(self):
        pass
